package paycard;

public class PayCardJunior extends PayCard {

    /*@ public static invariant juniorLimit==100 ;
      @*/

    /*@
      public instance invariant 
      this.balance >= 0 && this.balance < juniorLimit && juniorLimit < limit; 
      @*/

    private final static int juniorLimit=100;
    private int unsuccessfulOperations=0;

    /*@
      public normal_behavior
      requires cardLimit>juniorLimit;
also public normal_behavior
      requires cardLimit > 0;
     @*/
    public PayCardJunior(int cardLimit) {
		super(cardLimit);
    }

    /*@
      ensures \result.limit==100;
     @*/
    public static PayCardJunior createCard() {
		return new PayCardJunior(100);
    }

    /*@
      also
      public normal_behavior
      requires amount>0;
      ensures (\old(balance)+amount < juniorLimit)?
      (balance == \old(balance) + amount):
      ( balance == \old(balance) &&
        unsuccessfulOperations == \old(unsuccessfulOperations)+1);
     @*/
    public boolean charge(int amount) {
        try {
            charge0(amount);
	    return true;
        } catch (java.lang.Exception e) {
            this.unsuccessfulOperations++;
	    return false;
        }
    }

    /*@
      public exceptional_behavior
      requires amount <= 0 || checkSum(this.balance + amount) == 0;
      assignable \nothing;
      signals (CardException) amount <= 0 || checkSum(this.balance + amount) == 0;
also
      public normal_behavior
      requires amount>0 && checkSum(this.balance + amount) == 1;
      ensures (\old(balance)+amount < juniorLimit)?
      (balance == \old(balance) + amount):
      ( balance == \old(balance) &&
        unsuccessfulOperations == \old(unsuccessfulOperations)+1);
      @*/
    private void charge0(int amount) throws CardException {
	if(amount <= 0){
	    throw new CardException();
	} 
        int checkStatus = this.checkSum(this.balance + amount);
        if (checkStatus != 1) {
            throw new CardException();
        } else {
            this.balance = this.balance + amount;
        }
    }
    
    /*@
     private behavior
      ensures \result==1 ?  sum<juniorLimit : sum>=juniorLimit;
      ensures \result==1 || \result == 0;
      signals (Exception) false;
      @*/
    private /*@pure@*/ int checkSum(int sum) {
        if (sum >= this.juniorLimit) {
            return 0;
        } else {
            return 1;
        }
    }

    /*@ 
      public normal_behavior
       requires amount>0;
       ensures \old(balance)+amount<juniorLimit ?
       amount == balance - \old(balance) :
       ( balance == \old(balance) &&
       unsuccessfulOperations == \old(unsuccessfulOperations)+1);
     @*/
    public void complexCharge(int amount) {
        try {
            this.charge0(amount);
        } catch (CardException e) {
            this.unsuccessfulOperations++;
        }
    }
}
